from AccessControl      import getSecurityManager
from AccessControl.User import UserFolder, User
from OFS.Folder         import Folder
from OFS.SimpleItem     import SimpleItem
import Globals


class ZUserBase(User, SimpleItem):
    """Base class for all ZClasses of Users for use in the ZUserFolder"""

    meta_type='ZUserBase'

    manage_options = ( ( {'label' : 'Edit', 'action':'manage_edit',
                          #'help' : ('OFSP','ZUser-Folder_Contents.stx')
                          }, )
                       + SimpleItem.manage_options)

    __ac_permissions__=( ('Manage users',
                          ('manage_changeUser', 'manage_editUser', 'manage_editUserForm')),
                         )    

    def __creatable_by_emergency_user__(self):
        return 1

    #def __allow_access_to_unprotected_subobjects__(self, name, value=None): return 1

    def __init__(self, password, roles=[], domains=[]):
        #name parameter allows for id and name to unsync
        #so it is no longer
        User.__init__(self, '', password, roles, domains)

                
    def manage_changeUser(self, password, roles, domains):
        if password is not None:
            self.__=password
        self.roles=roles
        self.domains=domains


    manage_edit = Globals.DTMLFile('dtml/manage_edit',globals())
    
    def manage_editUser(self, password, confirm, roles, domains):
        """Change a user"""
        # sloppy, I know.  I'll get to that later.
        # SR: Nope, looks good to me.
        error = self.aq_parent._changeUser(self.name, password, confirm, roles, domains)
        if error:
            return error
        else:
            return Globals.MessageDialog(
                   title  ='User edited.',
                   message='The user has been updated.',
                   action ='%s/manage_edit' % self.absolute_url())

Globals.default__class_init__(ZUserBase)



class ZUserFolder(Folder, UserFolder):
    """Similar to Standard UserFolder object (but better)

    A UserFolder holds User objects which contain information
    about users including name, password domain, and roles.
    UserFolders function chiefly to control access by authenticating
    users and binding them to a collection of roles.

    A ZUserFolder allows the contained User objects to be ZClasses.
    Now you can access methods and properties of the AUTHENTICATED_USER
    that you have defined in the ZClass.
    """

    id        = 'acl_users'
    meta_type = 'Z User Folder'
    title     = 'Z User Folder'

    manage_options = (
        (
        {'label':'Contents', 'action':'manage_main',
         #'help':('OFSP','ZUser-Folder_Contents.stx')
         },
        )
        + SimpleItem.manage_options)

    __ac_permissions__=(
        ('Manage users',
         ('manage_users', 'manage_addUser', 'getUserNames', 'getUser', 'getUsers',
          'getUserById', 'user_names', 'setDomainAuthenticationMode',) ),
        )

    def filtered_meta_types(self, user=None):
        # Return a list of the types for which the user has
        # adequate permission to add that type of object.
            
        meta_types = Folder.inheritedAttribute('filtered_meta_types')(self, user)

        zclasses = {}
        zc = self.aq_acquire('_getProductRegistryData')('zclasses')
        for z in zc:
            zclasses[z['meta_type']] = z['meta_class']

        zuser_meta_types = []
        for meta_type in meta_types:
            zclass = zclasses.get(meta_type['name'], None)
            if zclass:
                try:
                    # to ignore problems with LoginManager classes
                    if 'ZUserBase' in zclass.ZClassBaseClassNames():
                        zuser_meta_types.append(meta_type)
                except:
                    pass

        return zuser_meta_types    

    
    def _setObject(self, id, object, roles=None, user=None, set_owner=1):
        # set the name so as to always sync
        object.name = id
        Folder.inheritedAttribute('_setObject')(self, id, object, roles, user, set_owner)


    def manage_addUser(self, name, user, REQUEST=None):
        ''' '''
        self._setObject(name, user)

        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def getUserNames(self):
        """Return a list of usernames"""
        return self.objectIds()


    def getUsers(self):
        """Return a list of user objects"""
        return self.objectValues()


    def getUser(self, name):
        """Return the named user object or None"""
        return getattr(self, name, None)


Globals.default__class_init__(ZUserFolder)



def manage_addZUserFolder(self,dtself=None,REQUEST=None,**ignored):
    '''manage_addZUserFolder(self,dtself=None,REQUEST=None,**ignored)'''
    
    zuf = ZUserFolder()

    self = self.this()
    try:
        self._setObject('acl_users', zuf)
    except:
        return Globals.MessageDialog(
            title  ='Item Exists',
            message='This object already contains a User Folder',
            action ='%s/manage_main' % REQUEST['URL1'])

    self.__allow_groups__ = zuf

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')








